@@ -16,13 +16,15 @@
 #'   their approximate standard errors.}
 #'  \item{vcov}{Variance-covariance matrix of parameter estimates.}
 #'  \item{log_Lik}{The log-likelihood value at convergence.}
-#'  \item{J}{The number of items.}
+#'  \item{N}{Number of units.}
+#'  \item{J}{Number of items.}
+#'  \item{H}{A vector denoting the number of response categories for each item.}
+#'  \item{ylevels}{A list showing the levels of the factorized response categories.}
 #'  \item{p}{The number of predictors for the mean equation.}
 #'  \item{q}{The number of predictors for the variance equation.}
-#'  \item{item_names}{Names of items.}
+#'  \item{control}{List of control values.}
 #'  \item{call}{The matched call.}
-#' @references Zhou, Xiang. 2017. "Hierarchical Item Response Models for Analyzing Public Opinion."
-#'   Working Paper.
+#' @references Zhou, Xiang. 2019. "\href{https://doi.org/10.1017/pan.2018.63}{Hierarchical Item Response Models for Analyzing Public Opinion.}" Political Analysis.
 #' @importFrom rms lrm.fit
 #' @importFrom pryr compose
 #' @importFrom pryr partial
@@ -54,10 +56,16 @@ hltm <- function(y, x, z, beta_set = 1, sign_set = TRUE, control = list()) {
     y <- as.data.frame(y)
     N <- nrow(y)
     J <- ncol(y)
-    for (j in seq(1, J)) y[[j]] <- fac2int(y[[j]]) - 1
+    yl <- vector(mode = "list", length = J)
+    for (j in seq(1, J)) {
+      tmp <- factor(y[[j]], exclude = c(NA, NaN))
+      yl[[j]] <- levels(tmp)
+      y[[j]] <- as.integer(tmp) - 1
+    }
     tmp <- match(TRUE, vapply(y, invalid_ltm, logical(1L)))
     if (!is.na(tmp))
       stop(paste(names(y)[tmp], "is not a dichotomous variable"))
+    H <- vapply(y, max, numeric(1L), na.rm = TRUE) + 1
 
     # check x and z (x and z should contain an intercept column)
     if (missing(x)) x <- as.matrix(rep(1, N))
@@ -175,8 +183,6 @@ hltm <- function(y, x, z, beta_set = 1, sign_set = TRUE, control = list()) {
         # check convergence
         if (sqrt(sum((beta - beta_prev)^2)) < con[["eps"]]) {
             cat("\n converged at iteration", iter, "\n")
-            gamma <- setNames(gamma, paste("x", colnames(x), sep = "_"))
-            lambda <- setNames(lambda, paste("z", colnames(z), sep = "_"))
             break
         } else if (iter == con[["max_iter"]]) {
             stop("algorithm did not converge; try increasing max_iter.")
@@ -184,6 +190,9 @@ hltm <- function(y, x, z, beta_set = 1, sign_set = TRUE, control = list()) {
         } else next
     }
 
+    gamma <- setNames(as.numeric(gamma), paste("x", colnames(x), sep = "_"))
+    lambda <- setNames(as.numeric(lambda), paste("z", colnames(z), sep = "_"))
+
     # inference
     pik <- matrix(unlist(Map(partial(dnorm, x = theta_ls), mean = fitted_mean,
                              sd = sqrt(fitted_var))), N, K, byrow = TRUE) * matrix(qw_ls, N, K, byrow = TRUE)
@@ -202,11 +211,10 @@ hltm <- function(y, x, z, beta_set = 1, sign_set = TRUE, control = list()) {
     s_ab <- unname(Reduce(cbind, lapply(1:J, sj_ab_ltm)))
 
     s_lambda <- s_gamma <- NULL
-    if (p > 1)
-        s_gamma <- vapply(1:N, si_gamma, numeric(p - 1))
-    if (q > 1)
-        s_lambda <- vapply(1:N, si_lambda, numeric(q - 1))
-    s_all <- rbind(t(s_ab), s_gamma, s_lambda)
+    s_gamma <- vapply(1:N, si_gamma, numeric(p))
+    s_lambda <- vapply(1:N, si_lambda, numeric(q))
+
+    s_all <- rbind(t(s_ab)[-c(1L, ncol(s_ab)), , drop = FALSE], s_gamma, s_lambda)
     s_all[is.na(s_all)] <- 0
     covmat <- solve(tcrossprod(s_all))
     se_all <- sqrt(diag(covmat))
@@ -214,17 +222,15 @@ hltm <- function(y, x, z, beta_set = 1, sign_set = TRUE, control = list()) {
     # reorganize se_all
     sH <- 2 * J
     lambda_indices <- gamma_indices <- NULL
-    if (p > 1)
-        gamma_indices <- (sH + 1):(sH + p - 1)
-    if (q > 1)
-        lambda_indices <- (sH + p):(sH + p + q - 2)
-    se_all <- c(se_all[1:sH], NA, se_all[gamma_indices], NA, se_all[lambda_indices])
+    gamma_indices <- (sH - 1):(sH + p - 2)
+    lambda_indices <- (sH + p - 1):(sH + p + q - 2)
+    se_all <- c(NA, se_all[1:(sH-2)], NA, se_all[gamma_indices], se_all[lambda_indices])
 
     # name se_all and covmat
     names_ab <- paste(rep(names(alpha), each = 2), c("Diff", "Dscrmn"))
     names(se_all) <- c(names_ab, names(gamma), names(lambda))
-    rownames(covmat) <- colnames(covmat) <- c(names_ab, names(gamma)[-1L],
-        names(lambda)[-1L])
+    rownames(covmat) <- colnames(covmat) <- c(names_ab[-c(1L, length(names_ab))], names(gamma),
+                                              names(lambda))
 
     # item coefficients
     coefs_item <- Map(function(a, b) c(Diff = a, Dscrmn = b), alpha, beta)
@@ -240,7 +246,7 @@ hltm <- function(y, x, z, beta_set = 1, sign_set = TRUE, control = list()) {
 
     # output
     out <- list(coefficients = coefs, scores = theta, vcov = covmat, log_Lik = log_Lik,
-        J = J, p = p, q = q, item_names = names(y), call = cl)
+                N = N, J = J, H = H, ylevels = yl, p = p, q = q, control = con, call = cl, inner = environment(theta_post_ltm))
     class(out) <- c("hltm", "hIRT")
     out
 }
